@@ -1,5 +1,5 @@
 /*  
- *  Copyright (c) 2012-2020, Peter Haag
+ *  Copyright (c) 2012-2021, Peter Haag
  *  All rights reserved.
  *  
  *  Redistribution and use in source and binary forms, with or without 
@@ -66,6 +66,8 @@
 #define GET_OPTION_TEMPLATE_FIELD_COUNT(p)		 (Get_val16((void *)((p) + 2)))
 #define GET_OPTION_TEMPLATE_SCOPE_FIELD_COUNT(p) (Get_val16((void *)((p) + 4)))
 
+#define CHECK_OPTION_DATA(avail, tag) ((tag.offset + tag.length) <= avail)
+
 #define DYN_FIELD_LENGTH	65535
 
 /* module limited globals */
@@ -97,13 +99,15 @@ typedef struct sequence_map_s {
 #define move_flags		14
 #define Time64Mili 		15
 #define TimeDeltaMicro 	16
-#define TimeUnix 		17
-#define saveICMP 		18
-#define zero8			19
-#define zero16			20
-#define zero32			21
-#define zero64			22
-#define zero128			23
+#define TimeMili		17
+#define SystemInitTime 	18
+#define TimeUnix 		19
+#define saveICMP 		20
+#define zero8			21
+#define zero16			22
+#define zero32			23
+#define zero64			24
+#define zero128			25
 
 	uint32_t	id;				// sequence ID as defined above
 	uint16_t	skip_count;		// skip this number of bytes in input stream after reading
@@ -131,6 +135,8 @@ typedef struct input_translation_s {
 	int			delta_time;				// delta micro or absolute ms time stamps
 	uint64_t	flow_start;				// start time in msec
 	uint64_t	flow_end;				// end time in msec
+	uint64_t	SysUpTime;				// System Init Time #160 per record
+	int			HasTimeMili;			// Exporter sent relative time stamps
 	uint32_t	icmpTypeCode;			// ICMP type/code in data stream
 	uint64_t    packets;				// total (in)packets - sampling corrected
 	uint64_t    bytes;					// total (in)bytes - sampling corrected
@@ -178,6 +184,10 @@ typedef struct exporterDomain_s {
 	uint64_t	TemplateRecords;	// stat counter
 	uint64_t	DataRecords;		// stat counter
 
+	// SysUptime if sent with #160
+	uint64_t	SysUpTime;			// in msec
+	optionTag_t SysUpOption;		// SysUpTime option information
+ 
 	// linked list of all templates sent by this exporter
 	input_translation_t	*input_translation_table; 
 
@@ -226,8 +236,8 @@ static struct ipfix_element_map_s {
 	{ IPFIX_bgpDestinationAsNumber, 	 _4bytes, 	_4bytes,  move32, zero32, EX_AS_4 },
 	{ IPFIX_bgpDestinationAsNumber, 	 _2bytes, 	_2bytes,  move16, zero16, EX_AS_2 },
 	{ IPFIX_bgpNextHopIPv4Address, 		 _4bytes, 	_4bytes,  move32, zero32, EX_NEXT_HOP_BGP_v4},
-	{ IPFIX_flowEndSysUpTime, 			 _4bytes, 	_4bytes,  nop, nop,  COMMON_BLOCK },
-	{ IPFIX_flowStartSysUpTime, 		 _4bytes, 	_4bytes,  nop, nop, COMMON_BLOCK },
+	{ IPFIX_flowEndSysUpTime,			 _4bytes,   _4bytes,  TimeMili, nop, COMMON_BLOCK},
+	{ IPFIX_flowStartSysUpTime, 		 _4bytes, 	_4bytes,  TimeMili, nop, COMMON_BLOCK },
 	{ IPFIX_postOctetDeltaCount, 		 _8bytes, 	_8bytes,  move64, zero64, EX_OUT_BYTES_8 },
 	{ IPFIX_postOctetDeltaCount, 		 _4bytes, 	_4bytes,  move32, zero32, EX_OUT_BYTES_4 },
 	{ IPFIX_postPacketDeltaCount, 		 _8bytes, 	_8bytes,  move64, zero64, EX_OUT_PKG_8 },
@@ -244,6 +254,8 @@ static struct ipfix_element_map_s {
 	{ IPFIX_vlanId, 					 _2bytes, 	_2bytes,  move16, zero16, EX_VLAN}, 
 	{ IPFIX_postVlanId, 				 _2bytes, 	_2bytes,  move16, zero16, EX_VLAN},
 	{ IPFIX_flowDirection, 				 _1byte, 	_1byte,   move8, zero8, EX_MULIPLE },
+	{ IPFIX_biflowDirection, 			 _1byte, 	_1byte,   move8, zero8, COMMON_BLOCK },
+	{ IPFIX_flowEndReason, 				 _1byte, 	_1byte,   move8, zero8, COMMON_BLOCK },
 	{ IPFIX_ipNextHopIPv6Address, 		 _16bytes, 	_16bytes, move128, zero128, EX_NEXT_HOP_v6},
 	{ IPFIX_bgpNextHopIPv6Address, 		 _16bytes, 	_16bytes, move128, zero128, EX_NEXT_HOP_BGP_v6},
 	{ IPFIX_mplsTopLabelStackSection, 	 _3bytes,   _4bytes,  move_mpls, zero32, EX_MPLS},
@@ -258,16 +270,29 @@ static struct ipfix_element_map_s {
 	{ IPFIX_mplsLabelStackSection10, 	 _3bytes,   _4bytes,  move_mpls, zero32, EX_MPLS},
 	{ IPFIX_DestinationMacAddress, 		 _6bytes,   _8bytes,  move_mac, zero64, EX_MAC_2},
 	{ IPFIX_postSourceMacAddress, 		 _6bytes,   _8bytes,  move_mac, zero64, EX_MAC_2},
-	{ IPFIX_flowStartMilliseconds, 		 _8bytes,   _8bytes,  Time64Mili, zero32, COMMON_BLOCK},
-	{ IPFIX_flowEndMilliseconds, 		 _8bytes,   _8bytes,  Time64Mili, zero32, COMMON_BLOCK},
+	{ IPFIX_flowStartMilliseconds, 		 _8bytes,   _8bytes,  Time64Mili, zero64, COMMON_BLOCK},
+	{ IPFIX_flowEndMilliseconds, 		 _8bytes,   _8bytes,  Time64Mili, zero64, COMMON_BLOCK},
 	{ IPFIX_flowStartSeconds, 		 	 _4bytes,   _4bytes,  TimeUnix, zero32, COMMON_BLOCK},
 	{ IPFIX_flowEndSeconds,  	 		 _4bytes,   _4bytes,  TimeUnix, zero32, COMMON_BLOCK},
-	{ IPFIX_flowEndMilliseconds, 		 _8bytes,   _8bytes,  Time64Mili, zero32, COMMON_BLOCK},
+	{ IPFIX_flowEndMilliseconds, 		 _8bytes,   _8bytes,  Time64Mili, zero64, COMMON_BLOCK},
 	{ IPFIX_flowStartDeltaMicroseconds,	 _4bytes,   _4bytes,  TimeDeltaMicro, zero32, COMMON_BLOCK},
 	{ IPFIX_flowEndDeltaMicroseconds, 	 _4bytes,   _4bytes,  TimeDeltaMicro, zero32, COMMON_BLOCK},
+	{ IPFIX_SystemInitTimeMiliseconds,	 _8bytes,   _8bytes,  SystemInitTime, nop, COMMON_BLOCK},
 	{0, 0, 0}
 };
 
+// map for corresponding reverse element, if enterprise ID = IPFIX_ReverseInformationElement
+static const struct ipfixReverseMap_s {
+	uint16_t ID;		// IPFIX element id 
+	uint16_t reverseID;	// reverse IPFIX element id
+} ipfixReverseMap[] = {
+	{ IPFIX_octetTotalCount, IPFIX_postOctetTotalCount},
+	{ IPFIX_packetTotalCount, IPFIX_postPacketTotalCount},
+	{ IPFIX_octetDeltaCount, IPFIX_postOctetDeltaCount},
+	{ IPFIX_packetDeltaCount, IPFIX_postPacketDeltaCount},
+	{ 0, 0},
+};
+
 // cache to be used while parsing a template
 static struct cache_s {
 	struct element_param_s {
@@ -310,7 +335,7 @@ static void remove_all_translation_tables(exporterDomain_t *exporter);
 
 static exporterDomain_t *GetExporter(FlowSource_t *fs, ipfix_header_t *ipfix_header);
 
-static uint32_t MapElement(uint16_t Type, uint16_t Length, uint32_t order);
+static uint32_t MapElement(uint16_t Type, uint16_t Length, uint32_t order, uint32_t EnterpriseNumber);
 
 static void PushSequence(input_translation_t *table, uint16_t Type, uint32_t *offset, void *stack);
 
@@ -365,6 +390,9 @@ int i;
 static int HasOptionTable(exporterDomain_t *exporter, uint16_t tableID ) {
 samplerOption_t *s;
 
+	if ( exporter->SysUpOption.length ) 
+		return 1;
+
 	s = exporter->samplerOption;
 	while ( s && s->tableID != tableID )
 		s = s->next;
@@ -434,16 +462,38 @@ uint32_t ObservationDomain = ntohl(ipfix_header->ObservationDomain);
 
 } // End of GetExporter
 
-static uint32_t MapElement(uint16_t Type, uint16_t Length, uint32_t order) {
-int	index;
+static uint32_t MapElement(uint16_t Type, uint16_t Length, uint32_t order, uint32_t EnterpriseNumber) {
 
-	cache.input_order[order].type	= Type;
+	cache.input_order[order].type 	= SKIP_ELEMENT;
 	cache.input_order[order].length	= Length;
 
-	index = cache.lookup_info[Type].index;
+	switch ( EnterpriseNumber ) {
+		case 0:		// no Enterprise value
+			break;
+		case 6871:	// yaf CERT Coordination Centre
+			dbg_printf(" Skip yaf CERT Coordination Centre\n");
+			return 0;
+			break;
+		case IPFIX_ReverseInformationElement:
+			for (int i=0; ipfixReverseMap[i].ID != 0; i++ ) {
+				if ( ipfixReverseMap[i].ID == Type ) {
+					Type = ipfixReverseMap[i].reverseID;
+					dbg_printf(" Reverse mapped element type: %u\n", Type);
+					break;
+				}
+			}
+			break;
+		default:
+			dbg_printf(" Skip enterprise id: %u\n", EnterpriseNumber);
+			return 0;
+	}
+
+
+	int index = cache.lookup_info[Type].index;
 	if ( index ) {
 		while ( index && ipfix_element_map[index].id == Type ) {
 			if ( Length == ipfix_element_map[index].length ) {
+				cache.input_order[order].type	= Type;
 				cache.lookup_info[Type].found  = 1;
 				cache.lookup_info[Type].length = Length;
 				cache.lookup_info[Type].index  = index;
@@ -459,7 +509,6 @@ int	index;
 		dbg_printf("Skip unknown element type: %u, Length: %u\n", Type, Length);
 	}
 
-	cache.input_order[order].type = SKIP_ELEMENT;
 	return 0;
 
 } // End of MapElement
@@ -874,6 +923,7 @@ size_t				size_required;
 	} else if ( cache.lookup_info[IPFIX_flowStartSysUpTime].found ) {
 		PushSequence( table, IPFIX_flowStartSysUpTime, NULL, &table->flow_start);
 		PushSequence( table, IPFIX_flowEndSysUpTime, NULL, &table->flow_end);
+		PushSequence( table, IPFIX_SystemInitTimeMiliseconds, NULL, &table->SysUpTime);
 		offset = BYTE_OFFSET_first + 8;
 		dbg_printf("Time stamp: flow start/end relative milliseconds: %u/%u\n", 
 			IPFIX_flowStartSysUpTime, IPFIX_flowEndSysUpTime);
@@ -895,8 +945,10 @@ size_t				size_required;
 	PushSequence( table, IPFIX_SourceTransportPort, &offset, NULL);
 	PushSequence( table, IPFIX_DestinationTransportPort, &offset, NULL);
 
-	// skip exporter_sysid and reserved
-	offset += 4;
+	// skip exporter_sysid
+	offset += 2;
+	PushSequence( table, IPFIX_biflowDirection, &offset, NULL);
+	PushSequence( table, IPFIX_flowEndReason, &offset, NULL);
 
 	/* IP address record
 	 * This record is expected in the output stream. If not available
@@ -1118,7 +1170,7 @@ sampler_t *sampler;
 		// no samplers so far 
 		sampler = (sampler_t *)malloc(sizeof(sampler_t));
 		if ( !sampler ) {
-			LogError( "Process_v9: Panic! malloc(): %s line %d: %s", __FILE__, __LINE__, strerror (errno));
+			LogError( "Process_ipfix: Panic! malloc(): %s line %d: %s", __FILE__, __LINE__, strerror (errno));
 			return;
 		}
 
@@ -1165,7 +1217,7 @@ sampler_t *sampler;
 				// end of sampler chain - insert new sampler
 				sampler->next = (sampler_t *)malloc(sizeof(sampler_t));
 				if ( !sampler->next ) {
-					LogError( "Process_v9: Panic! malloc(): %s line %d: %s", __FILE__, __LINE__, strerror (errno));
+					LogError( "Process_ipfix: Panic! malloc(): %s line %d: %s", __FILE__, __LINE__, strerror (errno));
 					return;
 				}
 				sampler = sampler->next;
@@ -1319,7 +1371,6 @@ int i;
 		NextElement 	 = (ipfix_template_elements_std_t *)ipfix_template_record->elements;
 		for ( i=0; i<count; i++ ) {
 			uint16_t Type, Length;
-			uint32_t ext_id;
 			int Enterprise;
 	
 			Type   = ntohs(NextElement->Type);
@@ -1327,17 +1378,7 @@ int i;
 			Enterprise = Type & 0x8000 ? 1 : 0;
 			Type = Type & 0x7FFF;
 
-			ext_id = MapElement(Type, Length, i);
-
-			// do we store this extension? enabled != 0
-			// more than 1 v9 tag may map to an extension - so count this extension once only
-			if ( ext_id && extension_descriptor[ext_id].enabled ) {
-				if ( cache.common_extensions[ext_id] == 0 ) {
-					cache.common_extensions[ext_id] = 1;
-					num_extensions++;
-				}
-			} 
-	
+			uint32_t EnterpriseNumber = 0;
 			if ( Enterprise ) {
 				ipfix_template_elements_e_t *e = (ipfix_template_elements_e_t *)NextElement;
 				size_required += 4;	// ad 4 for enterprise value
@@ -1347,7 +1388,8 @@ int i;
 					dbg_printf("ERROR: Not enough data for template elements! required: %i, left: %u", size_required, size_left);
 					return;
 				}
-				if ( ntohl(e->EnterpriseNumber) == IPFIX_ReverseInformationElement ) {
+				EnterpriseNumber = ntohl(e->EnterpriseNumber);
+				if ( EnterpriseNumber == IPFIX_ReverseInformationElement ) {
 					dbg_printf(" [%i] Enterprise: 1, Type: %u, Length %u Reverse Information Element: %u\n", i, Type, Length, ntohl(e->EnterpriseNumber));
 				} else {
 					dbg_printf(" [%i] Enterprise: 1, Type: %u, Length %u EnterpriseNumber: %u\n", i, Type, Length, ntohl(e->EnterpriseNumber));
@@ -1358,12 +1400,22 @@ int i;
 				dbg_printf(" [%i] Enterprise: 0, Type: %u, Length %u\n", i, Type, Length);
 				NextElement++;
 			}
+
+			// do we store this extension? enabled != 0
+			// more than 1 ipfix tag may map to an extension - so count this extension once only
+			uint32_t ext_id = MapElement(Type, Length, i, EnterpriseNumber);
+			if ( ext_id && extension_descriptor[ext_id].enabled ) {
+				if ( cache.common_extensions[ext_id] == 0 ) {
+					cache.common_extensions[ext_id] = 1;
+					num_extensions++;
+				}
+			} 
+
 		}
 
-		dbg_printf("Processed: %u\n", size_required);
-
+		dbg_printf("Processed: %u, num_extensions found: %u\n", size_required, num_extensions);
 		// compact input order and reorder sequencer
-		if ( compact_input_order() ) {
+		if ( num_extensions && compact_input_order() ) {
 			// valid template with common inout fields
 
 			// as the router IP address extension is not part announced in a template, we need to deal with it here
@@ -1630,6 +1682,13 @@ samplerOption_t *samplerOption;
 				samplerOption->interval.offset = offset;
 				SetFlag(samplerOption->flags, SAMPLER305);
 				break;
+
+			// SysUpTime information
+			case IPFIX_SystemInitTimeMiliseconds:
+				exporter->SysUpOption.length = length;
+				exporter->SysUpOption.offset = offset;
+				break;
+
 		}
 		offset += length;
 	}
@@ -1644,6 +1703,10 @@ samplerOption_t *samplerOption;
 		free(samplerOption);
 		dbg_printf("[%u] No Sampling information found\n", exporter->info.id);
 	}
+
+	if ( exporter->SysUpOption.length ) {
+		dbg_printf("[%u] SysupTime information found, offset: %u\n", exporter->info.id, exporter->SysUpOption.offset);
+	}
 	processed_records++;
 
 } // End of Process_ipfix_option_templates
@@ -1721,10 +1784,10 @@ char				*string;
 	  	data_record->ext_map	    = table->extension_info.map->map_id;
 		data_record->exporter_sysid = exporter->info.sysid;
 		data_record->nfversion		= 10;
-		data_record->reserved 		= 0;
 
 		table->flow_start 		    = 0;
 		table->flow_end 		    = 0;
+		table->SysUpTime			= 0;
 		table->packets 		  	    = 0;
 		table->bytes 		  	    = 0;
 		table->out_packets 	  	    = 0;
@@ -1738,7 +1801,8 @@ char				*string;
 
 			if ( input_offset > size_left ) {
 				// overrun
-				LogError("Process ipfix: buffer overrun!! input_offset: %i > size left data buffer: %u\n", input_offset, size_left);
+				LogError("Process ipfix: buffer overrun!! input_offset: %i > size left data buffer: %u", input_offset, size_left);
+				dbg_printf("Buffer overrun!! input_offset: %i > size left data buffer: %u\n", input_offset, size_left);
 				return;
 			} 
 
@@ -1842,6 +1906,15 @@ char				*string;
 					  *(uint64_t *)stack = ((1000000LL * (uint64_t)ExportTime) - DeltaMicroSec) / 1000LL;
 
 					} break;
+				case SystemInitTime:
+					{ uint64_t SysUpTime = Get_val64((void *)&in[input_offset]);
+					  *(uint64_t *)stack = SysUpTime;
+
+					} break;
+				case TimeMili:
+					*(uint32_t *)stack = Get_val32((void *)&in[input_offset]);
+					table->HasTimeMili = 1;
+					break;
 				case saveICMP:
 					*(uint32_t *)stack = Get_val16((void *)&in[input_offset]);
 					break;
@@ -1905,6 +1978,16 @@ char				*string;
 				*((uint32_t *)&out[table->received_offset+4]) = t.val.val32[1];
 		}
 
+		// record sysuptime overwrites option template sysuptime
+		if ( table->SysUpTime && table->HasTimeMili ) {
+			dbg_printf("Calculate first/last from record SysUpTime\n");
+			table->flow_start += table->SysUpTime;
+			table->flow_end	  += table->SysUpTime;
+		} else if ( exporter->SysUpTime && table->HasTimeMili ) {
+			dbg_printf("Calculate first/last from option SysUpTime\n");
+			table->flow_start += exporter->SysUpTime;
+			table->flow_end	  += exporter->SysUpTime;
+		}
 		// split first/last time into epoch/msec values
 		data_record->first 		= table->flow_start / 1000;
 		data_record->msec_first = table->flow_start % 1000;
@@ -1912,6 +1995,19 @@ char				*string;
 		data_record->last 		= table->flow_end / 1000;
 		data_record->msec_last	= table->flow_end % 1000;
 
+		/* cross check for invalid date/time records:
+		 * if either date is < 820454400 === 1.1.1996 (year of invention of netflow)
+		 * invalidates set start/end to 0
+		 */
+		if ( data_record->first < 820454400 || data_record->last < 820454400 ) {
+			dbg_printf("Zero date < 19960101\n");
+			data_record->first 		= 0;
+			data_record->msec_first = 0;
+			data_record->last 		= 0;
+			data_record->msec_last	= 0;
+			table->flow_start = 0;
+			table->flow_end	  = 0;
+		}
 		// update first_seen, last_seen
 		if ( table->flow_start < fs->first_seen )
 			fs->first_seen = table->flow_start;
@@ -2016,38 +2112,53 @@ char				*string;
 } // End of Process_ipfix_data
 
 static void  Process_ipfix_option_data(exporterDomain_t *exporter, void *data_flowset, FlowSource_t *fs) {
-samplerOption_t *samplerOption;
-uint32_t	tableID;
-uint8_t	 *in;
 
-	tableID  = GET_FLOWSET_ID(data_flowset);
+	uint32_t tableID   = GET_FLOWSET_ID(data_flowset);
+	uint32_t size_left = GET_FLOWSET_LENGTH(data_flowset) - 4; // -4 for data flowset header -> id and length
+	dbg_printf("[%u] Process option data flowset size: %u\n", exporter->info.id, size_left);
 
-	samplerOption = exporter->samplerOption;
+	// map input buffer as a byte array
+	uint8_t *in	= (uint8_t *)(data_flowset + 4);  // skip flowset header
+
+	if ( exporter->SysUpOption.length ) {
+		if (CHECK_OPTION_DATA(size_left, exporter->SysUpOption)) {
+			exporter->SysUpTime = Get_val(in, exporter->SysUpOption.offset, exporter->SysUpOption.length);
+			dbg_printf("Found SysUpTime option data\n");
+			dbg_printf("Extracted SysUpTime : %llu\n", exporter->SysUpTime);
+		} else {
+			LogError("Process_ipfix_option: %s line %d: Not enough data for option data", __FILE__, __LINE__);
+			return;
+		}
+	} else {
+		dbg_printf("No SysUpTime option data found\n");
+	}
+
+
+	samplerOption_t *samplerOption = exporter->samplerOption;
 	while ( samplerOption && samplerOption->tableID != tableID )
 		samplerOption = samplerOption->next;
 
 	if ( !samplerOption ) {
-		// should never happen - catch it anyway
-		LogError( "Process_ipfix: Panic! - No Offset table found! : %s line %d", __FILE__, __LINE__);
+		dbg_printf("No sampler option info\n");
 		return;
 	}
 
-#ifdef DEVEL
-	uint32_t size_left = GET_FLOWSET_LENGTH(data_flowset) - 4; // -4 for data flowset header -> id and length
-	dbg_printf("[%u] Process option data flowset size: %u\n", exporter->info.id, size_left);
-#endif
-
-	// map input buffer as a byte array
-	in	= (uint8_t *)(data_flowset + 4);  // skip flowset header
-
+	dbg_printf("sampler option found\n");
 	if ( (samplerOption->flags & SAMPLERMASK ) != 0) {
 		int32_t  id;
 		uint16_t mode;
 		uint32_t interval;
 
-		id	 = Get_val(in, samplerOption->id.offset, samplerOption->id.length);
-		mode = Get_val(in, samplerOption->mode.offset, samplerOption->mode.length);
-		interval = Get_val(in, samplerOption->interval.offset, samplerOption->interval.length);
+		if ( CHECK_OPTION_DATA(size_left, samplerOption->id) && 
+			 CHECK_OPTION_DATA(size_left, samplerOption->mode) &&
+			 CHECK_OPTION_DATA(size_left, samplerOption->interval)) {
+			id	 = Get_val(in, samplerOption->id.offset, samplerOption->id.length);
+			mode = Get_val(in, samplerOption->mode.offset, samplerOption->mode.length);
+			interval = Get_val(in, samplerOption->interval.offset, samplerOption->interval.length);
+		} else {
+			LogError("Process_ipfix_option: %s line %d: Not enough data for option data", __FILE__, __LINE__);
+			return;
+		}
 
 		InsertSampler(fs, exporter, id, mode, interval);
 
@@ -2063,8 +2174,14 @@ uint8_t	 *in;
 		uint32_t interval;
 
 		id		 = -1;
-		mode	 = Get_val(in, samplerOption->mode.offset, samplerOption->mode.length);
-		interval = Get_val(in, samplerOption->interval.offset, samplerOption->interval.length);
+		if ( CHECK_OPTION_DATA(size_left, samplerOption->mode) &&
+			 CHECK_OPTION_DATA(size_left, samplerOption->interval)) {
+			mode	 = Get_val(in, samplerOption->mode.offset, samplerOption->mode.length);
+			interval = Get_val(in, samplerOption->interval.offset, samplerOption->interval.length);
+		} else {
+			LogError("Process_ipfix_option: %s line %d: Not enough data for option data", __FILE__, __LINE__);
+			return;
+		}
 
  		InsertSampler(fs, exporter, id, mode, interval);
 
