@@ -1,5 +1,5 @@
 /*
- *  Copyright (c) 2009-2019, Peter Haag
+ *  Copyright (c) 2009-2020, Peter Haag
  *  Copyright (c) 2004-2008, SWITCH - Teleinformatikdienste fuer Lehre und Forschung
  *  All rights reserved.
  *  
@@ -51,18 +51,14 @@
 #include <stdint.h>
 #endif
 
+#include "util.h"
+#include "nfdump.h"
 #include "nffile.h"
 #include "nfx.h"
-#include "util.h"
+#include "exporter.h"
 #include "flist.h"
 #include "panonymizer.h"
 
-#if ( SIZEOF_VOID_P == 8 )
-typedef uint64_t    pointer_addr_t;
-#else
-typedef uint32_t    pointer_addr_t;
-#endif
-
 // module limited globals
 extension_map_list_t *extension_map_list;
 
@@ -180,7 +176,6 @@ nffile_t			*nffile_w;
 int 		i, done, ret, cnt, verbose;
 char		outfile[MAXPATHLEN], *cfile;
 
-	setbuf(stderr, NULL);
 	cnt 	= 1;
 	verbose = 1;
 
@@ -210,7 +205,7 @@ char		outfile[MAXPATHLEN], *cfile;
 		snprintf(outfile,MAXPATHLEN-1, "%s-tmp", cfile);
 		outfile[MAXPATHLEN-1] = '\0';
 		if ( verbose )
-			fprintf(stderr, " %i Processing %s\r", cnt++, cfile);
+			LogInfo(" %i Processing %s", cnt++, cfile);
 	}
 
 	if ( wfile )
@@ -273,7 +268,7 @@ char		outfile[MAXPATHLEN], *cfile;
 					LogError("(NULL) input file name error in %s line %d\n", __FILE__, __LINE__);
 					return;
 				}
-				LogError(" %i Processing %s\r", cnt++, cfile);
+				printf(" %i Processing %s\r", cnt++, cfile);
 
 				if ( wfile == NULL ) {
 					snprintf(outfile,MAXPATHLEN-1, "%s-tmp", cfile);
@@ -298,13 +293,8 @@ char		outfile[MAXPATHLEN], *cfile;
 				} break; // not really needed
 		}
 
-		if ( nffile_r->block_header->id == Large_BLOCK_Type ) {
-			// skip
-			continue;
-		}
-
 		if ( nffile_r->block_header->id != DATA_BLOCK_TYPE_2 ) {
-			fprintf(stderr, "Can't process block type %u. Skip block.\n", nffile_r->block_header->id);
+			LogError("Can't process block type %u. Skip block", nffile_r->block_header->id);
 			continue;
 		}
 
@@ -348,8 +338,8 @@ char		outfile[MAXPATHLEN], *cfile;
 							exit(255);
 					}
 					} break; 
-				case ExporterRecordType:
-				case SamplerRecordype:
+				case LegacyRecordType1:
+				case LegacyRecordType2:
 				case ExporterInfoRecordType:
 				case ExporterStatRecordType:
 				case SamplerInfoRecordype:
@@ -357,7 +347,7 @@ char		outfile[MAXPATHLEN], *cfile;
 					break;
 
 				default: {
-					fprintf(stderr, "Skip unknown record type %i\n", flow_record->type);
+					LogError("Skip unknown record type %i", flow_record->type);
 				}
 			}
 			// Advance pointer by number of bytes for netflow record
@@ -378,8 +368,7 @@ char		outfile[MAXPATHLEN], *cfile;
 
 	DisposeFile(nffile_w);
 
-	LogError("\n");
-	LogError("Processed %i files.\n", --cnt);
+	LogError("Processed %i files", --cnt);
 
 } // End of process_data
 
@@ -389,6 +378,7 @@ char 		*rfile, *Rfile, *wfile, *Mdirs;
 int			c;
 char		CryptoPAnKey[32];
 
+	memset((void *)CryptoPAnKey, 0, sizeof(CryptoPAnKey));
 	rfile = Rfile = Mdirs = wfile = NULL;
 	while ((c = getopt(argc, argv, "K:L:r:M:R:w:")) != EOF) {
 		switch (c) {
@@ -399,13 +389,13 @@ char		CryptoPAnKey[32];
 				break;
 			case 'K':
 				if ( !ParseCryptoPAnKey(optarg, CryptoPAnKey) ) {
-					fprintf(stderr, "Invalid key '%s' for CryptoPAn!\n", optarg);
+					LogError("Invalid key '%s' for CryptoPAn", optarg);
 					exit(255);
 				}
 				PAnonymizer_Init((uint8_t *)CryptoPAnKey);
 				break;
 			case 'L':
-				if ( !InitLog("argv[0]", optarg) )
+				if ( !InitLog(0, "argv[0]", optarg, 0) )
 					exit(255);
 				break;
 			case 'r':
@@ -428,12 +418,17 @@ char		CryptoPAnKey[32];
 		}
 	}
 
+	if ( CryptoPAnKey[0] == '\0' ) {
+		LogError("Expect -K <key> - 32 bytes key");
+		exit(255);
+	}
+
 	if ( rfile && Rfile ) {
-		fprintf(stderr, "-r and -R are mutually exclusive. Please specify either -r or -R\n");
+		LogError("-r and -R are mutually exclusive. Please specify either -r or -R");
 		exit(255);
 	}
 	if ( Mdirs && !(rfile || Rfile) ) {
-		fprintf(stderr, "-M needs either -r or -R to specify the file or file list. Add '-R .' for all files in the directories.\n");
+		LogError("-M needs either -r or -R to specify the file or file list. Add '-R .' for all files in the directories");
 		exit(255);
 	}
 
