@@ -1,8 +1,5 @@
 /*
- *  Copyright (c) 2017, Peter Haag
- *  Copyright (c) 2016, Peter Haag
- *  Copyright (c) 2014, Peter Haag
- *  Copyright (c) 2009, Peter Haag
+ *  Copyright (c) 2009-2020, Peter Haag
  *  Copyright (c) 2004-2008, SWITCH - Teleinformatikdienste fuer Lehre und Forschung
  *  All rights reserved.
  *  
@@ -357,6 +354,7 @@ void		*p = (void *)input_record;
 				output_record->event_flag = FW_EVENT;
 				output_record->fw_xevent  = tpl->fw_xevent;
 				output_record->icmp = tpl->nsel_icmp;
+				output_record->sec_group_tag = tpl->sec_group_tag;
 				p = (void *)tpl->data;
 			} break;
 			case EX_NSEL_XLATE_PORTS: {
@@ -499,7 +497,6 @@ int		i;
 	// write common record
 	size = COMMON_RECORD_DATA_SIZE;
 	memcpy((void *)common_record, (void *)master_record, size);
-	common_record->reserved = 0;
 	p = (void *)((pointer_addr_t)common_record + size);
 
 	// Required extension 1 - IP addresses
@@ -720,7 +717,7 @@ int		i;
 				tpl->fw_event   = master_record->event;
 				tpl->nsel_icmp  = master_record->icmp;
 				tpl->fill  = 0;
-				tpl->fill2 = 0;
+				tpl->sec_group_tag = master_record->sec_group_tag;
 				tpl->fw_xevent = master_record->fw_xevent;
 				p = (void *)tpl->data;
 				} break;
@@ -787,7 +784,7 @@ int		i;
 		}
 	}
 
-	nffile->block_header->size 		+= required;
+	nffile->block_header->size += required;
 	nffile->block_header->NumRecords++;
 #ifdef DEVEL
 	if ( ((pointer_addr_t)p - (pointer_addr_t)nffile->buff_ptr) != required ) {
