@@ -1,5 +1,5 @@
 /*
- *  Copyright (c) 2013-2019, Peter Haag
+ *  Copyright (c) 2013-2021, Peter Haag
  *  All rights reserved.
  *  
  *  Redistribution and use in source and binary forms, with or without 
@@ -83,20 +83,17 @@
 #include <pcap.h>
 
 #include "util.h"
+#include "nfdump.h"
 #include "nffile.h"
 #include "nfx.h"
-#include "nf_common.h"
+#include "expire.h"
 #include "nfnet.h"
 #include "flist.h"
 #include "nfstatfile.h"
 #include "bookkeeper.h"
 #include "collector.h"
 #include "exporter.h"
-#include "rbtree.h"
 #include "ipfrag.h"
-
-#include "expire.h"
-
 #include "flowtree.h"
 #include "netflow_pcap.h"
 #include "pcaproc.h"
@@ -149,6 +146,7 @@ typedef struct p_pcap_flush_thread_args_s {
 	// arguments
 	int		subdir_index;
 	char	*pcap_datadir;
+	char	*time_extension;
 	pcap_dev_t *pcap_dev; 
 	pcapfile_t *pcapfile;
 } p_pcap_flush_thread_args_t;
@@ -168,6 +166,7 @@ typedef struct p_packet_thread_args_s {
 	time_t	t_win;
 	int		subdir_index;
 	char	*pcap_datadir;
+	char	*time_extension;
 	int		live;
 } p_packet_thread_args_t;
 
@@ -183,6 +182,7 @@ typedef struct p_flow_thread_args_s {
 	NodeList_t *NodeList;		// pop new nodes from this list
 	FlowSource_t *fs;
 	time_t	t_win;
+	char	*time_extension;
 	int		subdir_index;
 	int		compress;
 	int		live;
@@ -199,7 +199,7 @@ static void Interrupt_handler(int sig);
 
 static void SetPriv(char *userid, char *groupid );
 
-static pcap_dev_t *setup_pcap_live(char *device, char *filter, int snaplen);
+static pcap_dev_t *setup_pcap_live(char *device, char *filter, int snaplen, int buffer_size);
 
 static pcap_dev_t *setup_pcap_Ffile(FILE *fp, char *filter, int snaplen);
 
@@ -226,6 +226,7 @@ static void usage(char *name) {
 					"-g groupid\tChange group to groupname\n"
 					"-i interface\tread packets from interface\n"
 					"-r pcapfile\tread packets from file\n"
+					"-b num\tset socket buffer size in MB. (default 20MB)\n"
 					"-B num\tset the node cache size. (default 524288)\n"
 					"-s snaplen\tset the snapshot length - default 1526\n"
 					"-e active,inactive\tset the active,inactive flow expire time (s) - default 300,60\n"
@@ -371,7 +372,7 @@ int		err;
 
 } // End of SetPriv
 
-static pcap_dev_t *setup_pcap_live(char *device, char *filter, int snaplen) {
+static pcap_dev_t *setup_pcap_live(char *device, char *filter, int snaplen, int buffer_size) {
 pcap_t 		*handle    = NULL;
 pcap_dev_t	*pcap_dev  = NULL;
 pcap_if_t	*alldevsp = NULL;
@@ -415,14 +416,43 @@ uint32_t	linkoffset, linktype;
 	 *  architectures that support it, you might want tune this value
 	 *  depending on how much traffic you're seeing on the network.
 	 */
-	handle = pcap_open_live(device, snaplen, PROMISC, TIMEOUT, errbuf);
-	if (handle == NULL) {
-		LogError("Couldn't open device %s: %s", device, errbuf);
+	handle = pcap_create(device, errbuf);
+	if ( !handle ) {
+		LogError("pcap_create() failed on %s: %s", device, errbuf);
 		return NULL;
 	}
 
-	// XXX
-	// int pcap_set_buffer_size(pcap_t *p, int buffer_size);
+	if ( pcap_set_snaplen(handle, snaplen)) { 
+		LogError("pcap_set_snaplen() failed: %s", pcap_geterr(handle));
+		pcap_close(handle);
+		return NULL;
+	}
+
+	if ( pcap_set_promisc(handle, PROMISC)) {
+		LogError("pcap_set_promisc() failed: %s", pcap_geterr(handle));
+		pcap_close(handle);
+		return NULL;
+	}
+
+	if ( pcap_set_timeout(handle, TIMEOUT) ) {
+		LogError("pcap_set_promisc() failed: %s", pcap_geterr(handle));
+		pcap_close(handle);
+		return NULL;
+	}
+
+	if ( buffer_size ) 
+		if ( pcap_set_buffer_size(handle, 1024 * 1024 * buffer_size) < 0 ) {
+			LogError("pcap_set_buffer_size() failed: %s", pcap_geterr(handle));
+			pcap_close(handle);
+			return NULL;
+		}
+	// else use platform default
+
+    if ( pcap_activate(handle) ) {
+		LogError("pcap_activate() failed: %s", pcap_geterr(handle));
+		pcap_close(handle);
+		return NULL;
+	}
 
 	if ( filter ) {
 		/* Compile and apply the filter */
@@ -568,21 +598,29 @@ uint32_t	linkoffset, linktype;
 } // End of setup_pcap_file
 
 static void SignalThreadTerminate(thread_info_t *thread_info, pthread_cond_t *thread_cond ) {
+struct timespec waitTime;
+
+	waitTime.tv_sec = 0;
+	waitTime.tv_nsec = 10000;
 
 	if ( !thread_info->done ) {
-		dbg_printf("Signal thread[%lu] to terminate\n", (long unsigned)thread_info->tid);
-    	if ( pthread_kill(thread_info->tid, SIGUSR2) != 0 ) {
-			dbg_printf("Failed to signal thread[%lu]\n", (long unsigned)thread_info->tid);
-		} 
-
-		// in case of a condition - signal condition
-		if ( thread_cond ) 
-			pthread_cond_signal(thread_cond);
+		do {
+			dbg_printf("Signal thread[%lu] to terminate\n", (long unsigned)thread_info->tid);
+    		if ( pthread_kill(thread_info->tid, SIGUSR2) != 0 ) {
+				dbg_printf("Failed to signal thread[%lu]\n", (long unsigned)thread_info->tid);
+			} 
+			nanosleep(&waitTime, NULL);
+		} while ( !thread_info->done );
 
 	} else {
 		dbg_printf("thread[%lu] gone already\n", (long unsigned)thread_info->tid);
 	}
 
+	// in case of a condition - signal condition
+	if ( thread_cond ) {
+		pthread_cond_signal(thread_cond);
+	}
+
    	if( pthread_join(thread_info->tid, NULL) == 0 ) {
        	dbg_printf("thread %lu joined\n", (long unsigned)thread_info->tid);
 	} else {
@@ -596,11 +634,12 @@ static void SignalThreadTerminate(thread_info_t *thread_info, pthread_cond_t *th
 __attribute__((noreturn)) static void *p_flow_thread(void *thread_data) {
 // argument dispatching
 p_flow_thread_args_t *args = (p_flow_thread_args_t *)thread_data;
-time_t t_win				 = args->t_win;
-int subdir_index			 = args->subdir_index;
-int compress			 	 = args->compress;
-int live			 	 	 = args->live;
-FlowSource_t *fs			 = args->fs;
+time_t t_win		 = args->t_win;
+int subdir_index	 = args->subdir_index;
+int compress	 	 = args->compress;
+int live		 	 = args->live;
+FlowSource_t *fs	 = args->fs;
+char *time_extension = args->time_extension;
 static time_t lastExpire = 0;
 time_t t_start, t_clock;
 int err, done;
@@ -664,7 +703,7 @@ int err, done;
 			char FullName[MAXPATHLEN];
 			char netflowFname[128];
 			char error[256];
-			char *subdir;
+			char *subdir, fmt[24];
 			uint32_t NumFlows;
 
 			// flush all flows to disk
@@ -675,6 +714,8 @@ int err, done;
 				NumFlows = Expire_FlowTree(fs, t_clock);
 
 			when = localtime(&t_start);
+			strftime(fmt, sizeof(fmt), time_extension, when);
+
 			nffile = fs->nffile;
 
 			// prepare sub dir hierarchy
@@ -686,17 +727,14 @@ int err, done;
 				
 					// failed to generate subdir path - put flows into base directory
 					subdir = NULL;
-					snprintf(netflowFname, 127, "nfcapd.%i%02i%02i%02i%02i",
-						when->tm_year + 1900, when->tm_mon + 1, when->tm_mday, when->tm_hour, when->tm_min);
+					snprintf(netflowFname, 127, "nfcapd.%s", fmt);
 				} else {
-					snprintf(netflowFname, 127, "%s/nfcapd.%i%02i%02i%02i%02i", subdir,
-						when->tm_year + 1900, when->tm_mon + 1, when->tm_mday, when->tm_hour, when->tm_min);
+					snprintf(netflowFname, 127, "%s/nfcapd.%s", subdir, fmt);
 				}
 
 			} else {
 				subdir = NULL;
-				snprintf(netflowFname, 127, "nfcapd.%i%02i%02i%02i%02i",
-					when->tm_year + 1900, when->tm_mon + 1, when->tm_mday, when->tm_hour, when->tm_min);
+				snprintf(netflowFname, 127, "nfcapd.%s", fmt);
 			}
 			netflowFname[127] = '\0';
 	
@@ -782,16 +820,21 @@ int err, done;
 			}
 		}
 
-		if ( Node->fin != SIGNAL_NODE ) {
-			// Process the Node
-			ProcessFlowNode(fs, Node);
-			time_t when = Node->t_last.tv_sec;
-			if ( (when - lastExpire) > EXPIREINTERVALL ) {
-				Expire_FlowTree(fs, when);
-				lastExpire = when;
-			} 
-			CacheCheck(fs, when, live);
-		}
+		time_t when;
+		if ( Node ) {
+			if ( Node->fin != SIGNAL_NODE ) {
+				// Process the Node
+				ProcessFlowNode(fs, Node);
+			}
+			when = Node->t_last.tv_sec;
+		} else {
+			when = time(NULL);
+		} 
+		if ( (when - lastExpire) > EXPIREINTERVALL ) {
+			Expire_FlowTree(fs, when);
+			lastExpire = when;
+		} 
+		CacheCheck(fs, when, live);
 
 	}
 
@@ -813,6 +856,7 @@ p_pcap_flush_thread_args_t *args = (p_pcap_flush_thread_args_t *)thread_data;
 char *pcap_datadir	 = args->pcap_datadir;
 pcap_dev_t *pcap_dev = args->pcap_dev;
 pcapfile_t *pcapfile = args->pcapfile;
+char *time_extension = args->time_extension;
 char pcap_dumpfile[MAXPATHLEN];
 int err;
 int runs = 0;
@@ -860,7 +904,7 @@ int runs = 0;
 			pcapfile->alternate_size = 0;
 		}
 
-		// if we are done, try to flsuh main data buffer
+		// if we are done, try to flush main data buffer
 		if ( args->done && pcapfile->data_size ) {
 			dbg_printf("Done: Flush all buffers\n");
 			// flush alternate buffer
@@ -877,11 +921,13 @@ int runs = 0;
 			char FullName[MAXPATHLEN];
 			char pcapFname[128];
 			char error[256];
-			char *subdir;
+			char *subdir, fmt[24];
 			int err;
 
 			dbg_printf("Flush rotate file\n");
 			when = localtime(&pcapfile->t_CloseRename);
+			strftime(fmt, sizeof(fmt), time_extension, when);
+
 			pcapfile->t_CloseRename = 0;
 
 			// prepare sub dir hierarchy
@@ -893,17 +939,14 @@ int runs = 0;
 				
 					// failed to generate subdir path - put flows into base directory
 					subdir = NULL;
-					snprintf(pcapFname, 127, "pcapd.%i%02i%02i%02i%02i",
-						when->tm_year + 1900, when->tm_mon + 1, when->tm_mday, when->tm_hour, when->tm_min);
+					snprintf(pcapFname, 127, "pcapd.%s", fmt);
 				} else {
-					snprintf(pcapFname, 127, "%s/pcapd.%i%02i%02i%02i%02i", subdir,
-						when->tm_year + 1900, when->tm_mon + 1, when->tm_mday, when->tm_hour, when->tm_min);
+					snprintf(pcapFname, 127, "%s/pcapd.%s", subdir, fmt);
 				}
 
 			} else {
 				subdir = NULL;
-				snprintf(pcapFname, 127, "pcapd.%i%02i%02i%02i%02i",
-					when->tm_year + 1900, when->tm_mon + 1, when->tm_mday, when->tm_hour, when->tm_min);
+				snprintf(pcapFname, 127, "pcapd.%s", fmt);
 			}
 			pcapFname[127] 	  = '\0';
 	
@@ -959,6 +1002,7 @@ p_packet_thread_args_t *args = (p_packet_thread_args_t *)thread_data;
 pcap_dev_t *pcap_dev = args->pcap_dev;
 time_t t_win		 = args->t_win;
 char *pcap_datadir 	 = args->pcap_datadir;
+char *time_extension = args->time_extension;
 int subdir_index	 = args->subdir_index;
 int live		 	 = args->live;
 // locals
@@ -994,13 +1038,14 @@ int err;
 			/* NOTREACHED */
 		}
 
-		p_flush_thread_args.done 	 	 = 0;
-		p_flush_thread_args.exit 	 	 = 0;
-		p_flush_thread_args.parent 	 	 = args->tid;
-		p_flush_thread_args.pcap_dev 	 = args->pcap_dev;
-		p_flush_thread_args.subdir_index = subdir_index;
-		p_flush_thread_args.pcap_datadir = pcap_datadir;
-		p_flush_thread_args.pcapfile 	 = pcapfile;
+		p_flush_thread_args.done		   = 0;
+		p_flush_thread_args.exit		   = 0;
+		p_flush_thread_args.parent		   = args->tid;
+		p_flush_thread_args.pcap_dev	   = args->pcap_dev;
+		p_flush_thread_args.subdir_index   = subdir_index;
+		p_flush_thread_args.pcap_datadir   = pcap_datadir;
+		p_flush_thread_args.time_extension = time_extension;
+		p_flush_thread_args.pcapfile	   = pcapfile;
 
 		err = pthread_create(&p_flush_thread_args.tid, NULL, p_pcap_flush_thread, (void *)&p_flush_thread_args);
 		if ( err ) {
@@ -1184,13 +1229,14 @@ int main(int argc, char *argv[]) {
 sigset_t			signal_set;
 struct sigaction	sa;
 int c, snaplen, err, do_daemonize;
-int subdir_index, compress, expire, cache_size;
+int subdir_index, compress, expire, cache_size, buff_size;
 int active, inactive;
 FlowSource_t	*fs;
 dirstat_t 		*dirstat;
 time_t 			t_win;
 char 			*device, *pcapfile, *filter, *datadir, *pcap_datadir, *extension_tags, pidfile[MAXPATHLEN], pidstr[32];
 char			*Ident, *userid, *groupid;
+char			*time_extension;
 pcap_dev_t 		*pcap_dev;
 p_packet_thread_args_t *p_packet_thread_args;
 p_flow_thread_args_t *p_flow_thread_args;
@@ -1200,7 +1246,7 @@ p_flow_thread_args_t *p_flow_thread_args;
 	launcher_pid	= 0;
 	device			= NULL;
 	pcapfile		= NULL;
-	filter			= NULL;
+	filter			= "ip";
 	pidfile[0]		= '\0';
 	t_win			= TIME_WINDOW;
 	datadir			= DEFAULT_DIR;
@@ -1209,14 +1255,16 @@ p_flow_thread_args_t *p_flow_thread_args;
 	Ident			= "none";
 	fs				= NULL;
 	extension_tags	= DefaultExtensions;
+	time_extension	= "%Y%m%d%H%M";
 	subdir_index	= 0;
 	compress		= NOT_COMPRESSED;
 	verbose			= 0;
 	expire			= 0;
 	cache_size		= 0;
+	buff_size		= 20;
 	active			= 0;
 	inactive		= 0;
-	while ((c = getopt(argc, argv, "B:DEI:e:g:hi:j:r:s:l:p:P:t:u:S:T:Vyz")) != EOF) {
+	while ((c = getopt(argc, argv, "B:DEI:b:e:g:hi:j:r:s:l:p:P:t:u:S:T:Vyz")) != EOF) {
 		switch (c) {
 			struct stat fstat;
 			case 'h':
@@ -1242,6 +1290,13 @@ p_flow_thread_args_t *p_flow_thread_args;
 			case 'I':
 				Ident = strdup(optarg);
 				break;
+			case 'b':
+				buff_size = atoi(optarg);
+				if (buff_size <= 0 || buff_size > 2047 ) {
+					LogError("ERROR: Buffer size in MB must be betwee 0..2047 (2GB max, default 20M)");
+					exit(EXIT_FAILURE);
+				}
+				break;
 			case 'i':
 				device = optarg;
 				break;
@@ -1302,13 +1357,13 @@ p_flow_thread_args_t *p_flow_thread_args;
 				} break;
 			case 't':
 				t_win = atoi(optarg);
-				if (t_win < 60) {
-					LogError("WARNING, very small time frame (< 60)!");
-				}
-				if (t_win <= 0) {
-					LogError("ERROR: time frame too small (<= 0)");
+				if (t_win < 2) {
+					LogError("time interval <= 2s not allowed");
 					exit(EXIT_FAILURE);
 				}
+				if (t_win < 60) {
+					time_extension	= "%Y%m%d%H%M%S";
+				}
 				break;
 			case 'j':
 				if ( compress ) {
@@ -1341,7 +1396,12 @@ p_flow_thread_args_t *p_flow_thread_args;
 						exit(255);
 					}
 					tmp[MAXPATHLEN-1] = 0;
-					snprintf(pidfile, MAXPATHLEN - 1 - strlen(tmp), "%s/%s", tmp, optarg);
+					if ( (strlen(tmp) + strlen(optarg) + 3) < MAXPATHLEN ) {
+						snprintf(pidfile, MAXPATHLEN - 3 - strlen(tmp), "%s/%s", tmp, optarg);
+					} else {
+						fprintf(stderr, "pidfile MAXPATHLEN error:\n");
+						exit(255);
+					}
 				}
 				// pidfile now absolute path
 				pidfile[MAXPATHLEN-1] = 0;
@@ -1389,6 +1449,11 @@ p_flow_thread_args_t *p_flow_thread_args;
 		exit(EXIT_FAILURE);
 	}
 
+	if ( !device && !pcapfile ) {
+		LogError("Specify either a device or a pcapfile to read packets from");
+		exit(EXIT_FAILURE);
+	}
+
 	if ( !Init_FlowTree(cache_size, active, inactive)) {
 		LogError("Init_FlowTree() failed.");
 		exit(EXIT_FAILURE);
@@ -1400,7 +1465,7 @@ p_flow_thread_args_t *p_flow_thread_args;
 	if ( pcapfile ) {
 		pcap_dev = setup_pcap_file(pcapfile, filter, snaplen);
 	} else {
-		pcap_dev = setup_pcap_live(device, filter, snaplen);
+		pcap_dev = setup_pcap_live(device, filter, snaplen, buff_size);
 	}
 	if (!pcap_dev) {
 		exit(EXIT_FAILURE);
@@ -1413,7 +1478,7 @@ p_flow_thread_args_t *p_flow_thread_args;
 		exit(255);
 	}
 
-	if ( do_daemonize && !InitLog(argv[0], SYSLOG_FACILITY)) {
+	if ( !InitLog(do_daemonize, argv[0], SYSLOG_FACILITY, verbose) ) {
 		pcap_close(pcap_dev->handle);
 		exit(255);
 	}
@@ -1529,13 +1594,14 @@ p_flow_thread_args_t *p_flow_thread_args;
 			__FILE__, __LINE__, strerror(errno) );
 		exit(255);
 	}	
-	p_flow_thread_args->fs 	   	   = fs;
-	p_flow_thread_args->t_win 	   = t_win;
-	p_flow_thread_args->compress   = compress;
-	p_flow_thread_args->live	   = device != NULL;
-	p_flow_thread_args->subdir_index = subdir_index;
-	p_flow_thread_args->parent	   = pthread_self();
-	p_flow_thread_args->NodeList   = NewNodeList();
+	p_flow_thread_args->fs			   = fs;
+	p_flow_thread_args->t_win		   = t_win;
+	p_flow_thread_args->compress	   = compress;
+	p_flow_thread_args->live		   = device != NULL;
+	p_flow_thread_args->subdir_index   = subdir_index;
+	p_flow_thread_args->parent		   = pthread_self();
+	p_flow_thread_args->NodeList	   = NewNodeList();
+	p_flow_thread_args->time_extension = time_extension;
 
 	err = 0;
 
@@ -1553,13 +1619,14 @@ p_flow_thread_args_t *p_flow_thread_args;
 			__FILE__, __LINE__, strerror(errno) );
 		exit(255);
 	}	
-	p_packet_thread_args->pcap_dev 	   = pcap_dev;
-	p_packet_thread_args->t_win 	   = t_win;
-	p_packet_thread_args->subdir_index = subdir_index;
-	p_packet_thread_args->pcap_datadir = pcap_datadir;
-	p_packet_thread_args->live 	   	   = device != NULL;
-	p_packet_thread_args->parent	   = pthread_self();
-	p_packet_thread_args->NodeList	   = p_flow_thread_args->NodeList;
+	p_packet_thread_args->pcap_dev		 = pcap_dev;
+	p_packet_thread_args->t_win			 = t_win;
+	p_packet_thread_args->subdir_index	 = subdir_index;
+	p_packet_thread_args->pcap_datadir	 = pcap_datadir;
+	p_packet_thread_args->live			 = device != NULL;
+	p_packet_thread_args->parent		 = pthread_self();
+	p_packet_thread_args->NodeList		 = p_flow_thread_args->NodeList;
+	p_packet_thread_args->time_extension = p_flow_thread_args->time_extension;
 
 	err = pthread_create(&p_packet_thread_args->tid, NULL, p_packet_thread, (void *)p_packet_thread_args);
 	if ( err ) {
