@@ -1,5 +1,5 @@
 /*
- *  Copyright (c) 2009-2020, Peter Haag
+ *  Copyright (c) 2009-2021, Peter Haag
  *  Copyright (c) 2004-2008, SWITCH - Teleinformatikdienste fuer Lehre und Forschung
  *  All rights reserved.
  *  
@@ -179,6 +179,10 @@ static void String_DstVlan(master_record_t *r, char *string);
 
 static void String_FwdStatus(master_record_t *r, char *string);
 
+static void String_BiFlowDir(master_record_t *r, char *string);
+
+static void String_FlowEndReason(master_record_t *r, char *string);
+
 static void String_Flags(master_record_t *r, char *string);
 
 static void String_InSrcMac(master_record_t *r, char *string);
@@ -326,6 +330,8 @@ static struct format_token_list_s {
 	{ "%smk", 0, "SMask", 				 	String_SrcMask },		// Src mask
 	{ "%dmk", 0, "DMask", 				 	String_DstMask },		// Dst mask
 	{ "%fwd", 0, "Fwd", 				 	String_FwdStatus },		// Forwarding Status
+	{ "%bfd", 0, "Bfd", 				 	String_BiFlowDir },		// BiFlow Direction
+	{ "%end", 0, "End", 				 	String_FlowEndReason },	// Flow End Reason
 	{ "%svln", 0, "SVlan", 				 	String_SrcVlan },		// Src Vlan
 	{ "%dvln", 0, "DVlan", 				 	String_DstVlan },		// Dst Vlan
 	{ "%ismc", 0, "  In src MAC Addr", 	 	String_InSrcMac },		// Input Src Mac Addr
@@ -1213,9 +1219,23 @@ static void String_FwdStatus(master_record_t *r, char *string) {
 
 } // End of String_FwdStatus
 
+static void String_BiFlowDir(master_record_t *r, char *string) {
+
+	snprintf(string, MAX_STRING_LENGTH-1 ,"%3u", r->biFlowDir);
+	string[MAX_STRING_LENGTH-1] = '\0';
+
+} // End of String_BiFlowDir
+
+static void String_FlowEndReason(master_record_t *r, char *string) {
+
+	snprintf(string, MAX_STRING_LENGTH-1 ,"%3u", r->flowEndReason);
+	string[MAX_STRING_LENGTH-1] = '\0';
+
+} // End of String_FlowEndReason
+
 static void String_Flags(master_record_t *r, char *string) {
 
-	snprintf(string, MAX_STRING_LENGTH-1 ,"%8s", FlagsString(r->tcp_flags));
+	snprintf(string, MAX_STRING_LENGTH-1 ,"%8s", FlagsString(r->prot == IPPROTO_TCP ? r->tcp_flags :0));
 	string[MAX_STRING_LENGTH-1] = '\0';
 
 } // End of String_Flags
