@@ -1,5 +1,5 @@
 /*
- *  Copyright (c) 2014-2019, Peter Haag
+ *  Copyright (c) 2014-2020, Peter Haag
  *  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
@@ -63,6 +63,8 @@
 
 #include <pcap.h>
 
+#include "util.h"
+#include "nfdump.h"
 #include "nffile.h"
 #include "bookkeeper.h"
 #include "collector.h"
@@ -70,7 +72,6 @@
 #include "ipfrag.h"
 #include "pcaproc.h"
 #include "content_dns.h"
-#include "util.h"
 #include "netflow_pcap.h"
 
 struct pcap_timeval {
@@ -134,13 +135,18 @@ pcapfile_t *OpenNewPcapFile(pcap_t *p, char *filename, pcapfile_t *pcapfile) {
 	}
 
 	if ( filename ) {
-		pcapfile->pd = pcap_dump_open(p, filename);
+		FILE* pFile = fopen(filename, "wb"); 
+		if ( !pFile ) {
+			LogError("fopen() error in %s line %d: %s", __FILE__, __LINE__, strerror(errno) );
+			return NULL;
+		}
+		pcapfile->pd = pcap_dump_fopen(p, pFile);
 		if ( !pcapfile->pd ) {
 			LogError("Fatal: pcap_dump_open() failed for file '%s': %s", filename, pcap_geterr(p));
 			return NULL;
 		} else {
-			fflush((FILE *)pcapfile->pd);
-			pcapfile->pfd = fileno((FILE *)pcapfile->pd);
+			fflush(pFile);
+			pcapfile->pfd = fileno((FILE *)pFile);
 			return pcapfile;
 		}
 	} else
@@ -151,10 +157,7 @@ pcapfile_t *OpenNewPcapFile(pcap_t *p, char *filename, pcapfile_t *pcapfile) {
 int ClosePcapFile(pcapfile_t *pcapfile) {
 int err = 0;
 
-	if ( fclose((FILE *)pcapfile->pd) < 0 ) {
-		LogError("close() error in %s line %d: %s", __FILE__, __LINE__, strerror(errno) );
-		err = errno;
-	}
+	pcap_dump_close(pcapfile->pd);
 	pcapfile->pfd = -1;
 
 	return err;
@@ -202,13 +205,6 @@ void PcapDump(pcapfile_t *pcapfile,  struct pcap_pkthdr *h, const u_char *sp) {
 struct pcap_sf_pkthdr sf_hdr;
 size_t	size = sizeof(struct pcap_sf_pkthdr) + h->caplen;
 
-/*
-	if ( pcapfile->pd)
-		pcap_dump((u_char *)pcapfile->pd, h, sp);
-	else
-		printf("NULL handle\n");
-	return;
-*/
 	if ( (pcapfile->data_size + size ) > BUFFSIZE ) {
 		void *_b;
 		// no space left in buffer - rotate buffers
@@ -342,16 +338,32 @@ static inline void ProcessICMPFlow(FlowSource_t	*fs, struct FlowNode *NewNode )
 } // End of ProcessICMPFlow
 
 static inline void ProcessOtherFlow(FlowSource_t *fs, struct FlowNode *NewNode ) {
+struct FlowNode *Node;
 
-	// Flush Other packets directly
-	StorePcapFlow(fs, NewNode);
-	dbg_printf("Flush Other flow: Proto: %u, Packets: %u, Bytes: %u\n", NewNode->proto, NewNode->packets, NewNode->bytes);
+	assert(NewNode->memflag == NODE_IN_USE);
+
+	// insert ESP/AH traffic
+	Node = Insert_Node(NewNode);
+	// if insert fails, the existing node is returned -> flow exists already
+	if ( Node == NULL ) {
+		dbg_printf("New flow IP proto: %u. Packets: %u, Bytes: %u\n",
+			NewNode->proto, NewNode->packets, NewNode->bytes);
+		return;
+	}
+	assert(Node->memflag == NODE_IN_USE);
+
+	// update existing flow
+	Node->packets++;
+	Node->bytes += NewNode->bytes;
+	Node->t_last = NewNode->t_last;
+	dbg_printf("Existing flow IP proto: %. Packets: %u, Bytes: %u\n",
+		NewNode->proto, Node->packets, Node->bytes);
 
 	Free_Node(NewNode);
 
-
 } // End of ProcessOtherFlow
 
+
 void ProcessFlowNode(FlowSource_t *fs, struct FlowNode *node) {
 
 	switch (node->proto) {
@@ -387,20 +399,13 @@ static unsigned pkg_cnt = 0;
 	pcap_dev->proc_stat.packets++;
 	offset = pcap_dev->linkoffset;
 	defragmented = NULL;
-
-	Node = New_Node();
-	if ( !Node ) {
-		pcap_dev->proc_stat.skipped++;
-		LogError("Node allocation error - skip packet");
-		return;
-	}
+	Node = NULL;
 
 	if ( pcap_dev->linktype == DLT_EN10MB ) {
 		ethertype = data[12] << 0x08 | data[13];
 		int	IEEE802 = ethertype <= 1500;
 		if ( IEEE802 ) {
 			pcap_dev->proc_stat.skipped++;
-			Free_Node(Node);
 			return;
 		}
 		REDO_LINK:
@@ -414,61 +419,66 @@ static unsigned pkg_cnt = 0;
 						dbg_printf("VLAN ID: %u, type: 0x%x\n",
 							ntohs(vlan_hdr->vlan_id), ntohs(vlan_hdr->type) );
 						ethertype = ntohs(vlan_hdr->type);
-/*
-pkt->vlans[pkt->vlan_count].pcp = (p[0] >> 5) & 7;
-	  pkt->vlans[pkt->vlan_count].cfi = (p[0] >> 4) & 1;
-	  pkt->vlans[pkt->vlan_count].vid = uint_16_be(p) & 0xfff;
-*/
 						offset += 4;
 					} while ( ethertype == 0x8100 );
 			
 					// redo ethertype evaluation
 					goto REDO_LINK;
 					} break;
-				case 0x806:	 // skip ARP
-					// silently skip ARP
-					pcap_dev->proc_stat.skipped++;
-					goto END_FUNC;
-					break;
+				case 0x8847: { // MPLS
+					// unwind MPLS label stack
+					uint32_t *mpls = (uint32_t *)(data + offset);
+					offset += 4;
+					dbg_printf("MPLS label: %x\n", ntohl(*mpls) >> 8);
+					while ((offset < hdr->caplen) && ((ntohl(*mpls) & 0x100) == 0)) { // check for Bottom of stack
+						offset += 4;
+						mpls++;
+						dbg_printf("MPLS label: %x\n", ntohl(*mpls) >> 8);
+					}
+					uint8_t *hdr = (uint8_t *)data + offset;
+					if((*hdr >> 4) == 4)
+						ethertype = 0x0800;	// IPv4
+					else if((*hdr >> 4) == 6)
+						ethertype = 0x86DD;	// IPv6
+					else {
+						LogInfo("Unsupported protocol: 0x%x", *hdr >> 4);
+						goto END_FUNC;
+					}
+
+					// redo ethertype evaluation
+					goto REDO_LINK;
+					} break;
 				case 0x26:	 // ?? multicast router termination ??
+				case 0x32:	
+				case 0x806:	 // skip ARP
 				case 0x4305: // B.A.T.M.A.N. BATADV
 				case 0x886f: // MS NLB heartbeat
 				case 0x88a2: // ATA over ethernet
 				case 0x88cc: // CISCO LLDP
 				case 0x9000: // Loop
+				case 0x9003: 
+				case 0x8808: // Ethernet flow control
 				case 0x880b: // PPP - rfc 7042
 				case 0x6558: // Ethernet Bridge
 					pcap_dev->proc_stat.skipped++;
-					if ( Node->proto ) {
-						// if it's an encap which we do not understand yet - push tunnel
-						Push_Node(NodeList, Node);
-					} else {
-						pcap_dev->proc_stat.skipped++;
-						dbg_printf("Skip Ethertype 0x%x", ethertype);
-						Free_Node(Node);
-					}
 					goto END_FUNC;
 					break;
 				default:
 					pcap_dev->proc_stat.unknown++;
-					LogInfo("Unsupported link type: 0x%x, packet: %u", ethertype, pkg_cnt);
-					Free_Node(Node);
+					LogInfo("Unsupported ether type: 0x%x, packet: %u", ethertype, pkg_cnt);
 					goto END_FUNC;
 			}
+	} else if ( pcap_dev->linktype != DLT_RAW ) { // we can still process raw IP
+		LogInfo("Unsupported link type: 0x%x, packet: %u", pcap_dev->linktype, pkg_cnt);
+		return;
 	}
 
 	if (hdr->caplen < offset) {
 		pcap_dev->proc_stat.short_snap++;
 		LogInfo("Short packet: %u/%u", hdr->caplen, offset);
-		Free_Node(Node);
 		goto END_FUNC;
 	}
 
-	Node->t_first.tv_sec = hdr->ts.tv_sec;
-	Node->t_first.tv_usec = hdr->ts.tv_usec;
-	Node->t_last.tv_sec  = hdr->ts.tv_sec;
-	Node->t_last.tv_usec  = hdr->ts.tv_usec;
-
 	data	 = data + offset;
 	data_len = hdr->caplen - offset;
 	offset	 = 0;
@@ -481,13 +491,13 @@ pkt->vlans[pkt->vlan_count].pcp = (p[0] >> 5) & 7;
 		// REDO loop could result in a memory leak, if again IP is fragmented
 		// XXX memory leak to be fixed
 		LogError("Fragmentation memory leak triggered!");
+		goto END_FUNC;
 	}
 
 	ip  	= (struct ip *)(data + offset); // offset points to end of link layer
 	version = ip->ip_v;	 // ip version
 
 	if ( version == 6 ) {
-		uint64_t *addr;
 		struct ip6_hdr *ip6 = (struct ip6_hdr *) (data + offset);
 		size_ip = sizeof(struct ip6_hdr);
 		offset = size_ip;	// offset point to end of IP header
@@ -496,7 +506,6 @@ pkt->vlans[pkt->vlan_count].pcp = (p[0] >> 5) & 7;
 			LogInfo("Packet: %u Length error: data_len: %u < size IPV6: %u, captured: %u, hdr len: %u",
 				pkg_cnt, data_len, size_ip, hdr->caplen, hdr->len);	
 			pcap_dev->proc_stat.short_snap++;
-			Free_Node(Node);
 			goto END_FUNC;
 		}
 
@@ -515,11 +524,25 @@ pkt->vlans[pkt->vlan_count].pcp = (p[0] >> 5) & 7;
 
 		payload = (void *)ip + size_ip;
 
-		addr = (uint64_t *)&ip6->ip6_src;
+		Node = New_Node();
+		if ( !Node ) {
+			pcap_dev->proc_stat.skipped++;
+			LogError("Node allocation error - skip packet");
+			return;
+		}
+		Node->t_first.tv_sec = hdr->ts.tv_sec;
+		Node->t_first.tv_usec = hdr->ts.tv_usec;
+		Node->t_last.tv_sec  = hdr->ts.tv_sec;
+		Node->t_last.tv_usec  = hdr->ts.tv_usec;
+
+		// keep compiler happy - get's optimized out anyway
+		void *p = (void *)&ip6->ip6_src;
+		uint64_t *addr = (uint64_t *)p;
 		Node->src_addr.v6[0] = ntohll(addr[0]);
 		Node->src_addr.v6[1] = ntohll(addr[1]);
 
-		addr = (uint64_t *)&ip6->ip6_dst;
+		p = (void *)&ip6->ip6_dst;
+		addr = (uint64_t *)p;
 		Node->dst_addr.v6[0] = ntohll(addr[0]);
 		Node->dst_addr.v6[1] = ntohll(addr[1]);
 		Node->version = AF_INET6;
@@ -534,7 +557,6 @@ pkt->vlans[pkt->vlan_count].pcp = (p[0] >> 5) & 7;
 			LogInfo("Packet: %u Length error: data_len: %u < size IPV4: %u, captured: %u, hdr len: %u",
 				pkg_cnt, data_len, size_ip, hdr->caplen, hdr->len);	
 			pcap_dev->proc_stat.short_snap++;
-			Free_Node(Node);
 			goto END_FUNC;
 		}
 
@@ -575,7 +597,6 @@ pkt->vlans[pkt->vlan_count].pcp = (p[0] >> 5) & 7;
 			if ( defragmented == NULL ) {
 				// not yet complete
 				dbg_printf("Fragmentation not yet completed. Size %u bytes\n", payload_len);
-				Free_Node(Node);
 				goto END_FUNC;
 			}
 			dbg_printf("Fragmentation complete\n");
@@ -584,6 +605,17 @@ pkt->vlans[pkt->vlan_count].pcp = (p[0] >> 5) & 7;
 		}
 		bytes 		= payload_len;
 
+		Node = New_Node();
+		if ( !Node ) {
+			pcap_dev->proc_stat.skipped++;
+			LogError("Node allocation error - skip packet");
+			return;
+		}
+		Node->t_first.tv_sec = hdr->ts.tv_sec;
+		Node->t_first.tv_usec = hdr->ts.tv_usec;
+		Node->t_last.tv_sec  = hdr->ts.tv_sec;
+		Node->t_last.tv_usec  = hdr->ts.tv_usec;
+
 		Node->src_addr.v6[0] = 0;
 		Node->src_addr.v6[1] = 0;
 		Node->src_addr.v4 = ntohl(ip->ip_src.s_addr);
@@ -593,9 +625,8 @@ pkt->vlans[pkt->vlan_count].pcp = (p[0] >> 5) & 7;
 		Node->dst_addr.v4 = ntohl(ip->ip_dst.s_addr);
 		Node->version = AF_INET;
 	} else {
-		LogInfo("ProcessPacket() Unsupprted protocol version: %i", version);
+		LogInfo("ProcessPacket() Unsupported protocol version: %i", version);
 		pcap_dev->proc_stat.unknown++;
-		Free_Node(Node);
 		goto END_FUNC;
 	}
 
@@ -778,9 +809,8 @@ pkt->vlans[pkt->vlan_count].pcp = (p[0] >> 5) & 7;
 
 			} break;
 		default:
-			// not handled protocol - simply save node
+			// other IP protocols are treated as stream such as ESP etc.
 			Push_Node(NodeList, Node);
-			pcap_dev->proc_stat.unknown++;
 			break;
 	}
 
